from pypika import Query, Table, Columns, AliasedQuery, Case
from pypika.functions import  Sum, Min, NullIf, Cast
from pypika.analytics import Rank


class FaveReservations(Table):

    def __init__(self):
        super().__init__(name = 'fave_reservations', schema = 'bi_dwh')


        # Basic Metrics ---------------------------------------------------
        self.cancellation_gmv = (
            Case()
            .when(self.reservation_status.like('%Cancel%'), self.selling_price)
            .else_(0)
        )

        self.first_time_merchant = (
            Case()
            .when(
                Rank()
                .over(
                    self.user_id,
                    self.schema_name,
                    self.company_id
                )
                .orderby(
                    self.reservation_date_lt
                ) == 1, 
                True
            )
            .else_(None)
        )

        self.first_time_outlet = (
            Case()
            .when(
                Rank()
                .over(
                    self.user_id,
                    self.schema_name,
                    self.outlet_id
                )
                .orderby(
                    self.reservation_date_lt
                ) == 1, 
                True
            )
            .else_(None)
        )

        # USD Metrics -----------------------------------------------------
        self.selling_price_usd = (
            self.selling_price / self.to_usd_exc_rate_on_rsv_date 
        )
        self.gross_revenue_usd = (
            self.gross_revenue / self.to_usd_exc_rate_on_rsv_date 
        )
        self.gross_profit_usd = (
            self.gross_profit / self.to_usd_exc_rate_on_rsv_date 
        )
        self.gross_profit_granted_usd = (
            self.gross_profit_granted / self.to_usd_exc_rate_on_rsv_date 
        )
