from fireant import (
    DataSet,
    Field,
    DataType,
    Join
)
from ..database import rds_warehouse
from ..bi_dwh import (
    fave_reservations as fr,
    all_companies as ac,
    all_outlets as ao, 
    user_profiling as up,
    master_partner_credit_grants as pcg
) 
from pypika import JoinType
from pypika.functions import Sum, Count
from ..custom_functions import DatePart


fr2 = fr.as_('fr2')

overall = DataSet(
    database=rds_warehouse,
    table=fr,
    joins=[
        Join(
            ac, 
            (ac.id == fr.company_id) &
            (ac.schema_name == fr.schema_name),
            JoinType.left
        ),
        Join(
            ao, 
            (ao.id == fr.outlet_id) &
            (ao.schema_name == fr.schema_name),
            JoinType.left
        ),
        Join(
            up, 
            up.user_id == fr.user_id,
            JoinType.left
        ),
        # For Customer Cohort Analysis
        Join(
            fr2,
            (fr2.user_id == fr.user_id) &
            (fr2.reservation_date_lt >= fr.reservation_date_lt),
        ),
        Join(
            pcg,
            (pcg.schema_name == fr.schema_name) & 
            (pcg.transactable_id == fr.reservation_set_id) & 
            (pcg.transactable_tyep == 'FavePayment') &
            (fr.reservation_type == 'FP')
        )
    ],
    fields=[
        Field(
            alias='reservation_id',
            definition=fr.reservation_id,
            label='id'
        ),
        Field(
            alias='schema_name',
            definition=fr.schema_name,
            data_type=DataType.text,
            label='schema_name'
        ),
        Field(
            alias='country',
            definition=fr.country,
            data_type=DataType.text,
            label='country'
        ),
        Field(
            alias='reservation_date_lt',
            definition=fr.reservation_date_lt,
            data_type=DataType.date,
            label='Transaction Date'
        ),
        Field(
            alias='cancellation_date_lt',
            definition=fr.cancellation_date_lt,
            data_type=DataType.date,
            label='Cancellation Date'
        ),
        Field(
            alias='redeemed_at_lt',
            definition=fr.redeemed_at_lt,
            data_type=DataType.date,
            label='Redeemed At'
        ),
        Field(
            alias='reservation_type',
            definition=fr.reservation_type,
            data_type=DataType.text,
            label='Reservation Type'
        ),
        Field(
            alias='reservation_status',
            definition=fr.reservation_status,
            data_type=DataType.text,
            label='Reservation Status'
        ),
        Field(
            alias='payment_type',
            definition=fr.payment_type,
            data_type=DataType.text,
            label='Payment_Type'
        ),
        Field(
            alias='activity_id',
            definition=fr.activity_id,
            label='Deal ID'
        ),
        Field(
            alias='activity_name',
            definition=fr.activity_name,
            data_type=DataType.text,
            label='Deal Name'
        ),
        Field(
            alias='user_id',
            definition=fr.user_id,
            label='User ID'
        ),
        Field(
            alias='bd_manager_id',
            definition=fr.bd_manager_id,
            label='BD Manager ID'
        ),
        Field(
            alias='name',
            definition=fr.name,
            data_type=DataType.text,
            label='BD Manager Name'
        ),
        Field(
            alias='company_id',
            definition=fr.company_id,
            label='Company ID'
        ),
        Field(
            alias='company_name',
            definition=fr.company_name,
            data_type=DataType.text,
            label='Company Name'
        ),
        Field(
            alias='outlet_id',
            definition=fr.outlet_id,
            label='Outlet ID'
        ),
        Field(
            alias='outlet_name',
            definition=fr.outlet_name,
            data_type=DataType.text,
            label='Outlet Name'
        ),
        Field(
            alias='outlet_city',
            definition=fr.outlet_city,
            data_type=DataType.text,
            label='Outlet City'
        ),
        Field(
            alias='reservation_mall_id',
            definition=fr.reservation_mall_id,
            label='Reservation Mall ID'
        ),
        Field(
            alias='reservation_mall_name',
            definition=fr.reservation_mall_name,
            data_type=DataType.text,
            label='Reservation Mall Name'
        ),
        Field(
            alias='redemptiom_outlet_id',
            definition=fr.redemptiom_outlet_id,
            label='Redemption Outlet ID'
        ),
        Field(
            alias='redemptiom_mall_id',
            definition=fr.redemptiom_mall_id,
            label='Redemption Mall ID'
        ),
        Field(
            alias='redemption_mall_name',
            definition=fr.redemption_mall_name,
            data_type=DataType.text,
            label='Redemption Mall Name'
        ),
        Field(
            alias='main_category',
            definition=fr.main_category,
            data_type=DataType.text,
            label='Main Category'
        ),
        Field(
            alias='value_currency',
            definition=fr.value_currency,
            data_type=DataType.text,
            label='Value Currency'
        ),
        # Boolean -------------------------------------------
        Field(
            alias='first_time',
            definition=fr.first_time,
            data_type=DataType.boolean,
            label='First Time'
        ),
        Field(
            alias='first_time_rs',
            definition=fr.first_time_rs,
            data_type=DataType.boolean,
            label='First Time Deals'
        ),
        Field(
            alias='first_time_fp',
            definition=fr.first_time_fp,
            data_type=DataType.boolean,
            label='First Time FavePay'
        ),
        Field(
            alias='first_time_e_card',
            definition=fr.first_time_e_card,
            data_type=DataType.boolean,
            label='First Time eCard'
        ),
        # Analystics Metrics --------------------------------
        Field(
            alias='transactions',
            definition=Count(fr.reservation_set_id).distinct(),
            label='Transactions'
        ),
        Field(
            alias='voucher_sold',
            definition=Count(fr.reservation_id).distinct(),
            label='Voucher Sold'
        ),
        Field(
            alias='active_customers',
            definition=Count(fr.user_id).distinct(),
            label='Active Customers'
        ),
        Field(
            alias='new_customers',
            definition=Count(fr.first_time),
            label='New Customers'
        ),
        Field(
            alias='returning_customers',
            definition=Count(fr.user_id).distinct() - Count(fr.first_time),
            label='Returning Customers'
        ),
        Field(
            alias='selling_price',
            definition=Sum(fr.selling_price),
            label='Total GMV',
            precision=2,
            thousands=','
        ),
        Field(
            alias='gross_revenue',
            definition=Sum(fr.gross_revenue),
            label='Gross Revenue',
            precision=2,
            thousands=','
        ),
        Field(
            alias='gross_profit',
            definition=Sum(fr.gross_profit),
            label='Gross Profit',
            precision=2,
            thousands=','
        ),
        Field(
            alias='gross_profit_granted',
            definition=Sum(fr.gross_profit_granted),
            label='Gross Profit Granted',
            precision=2,
            thousands=','
        ),
        # Analystics Metrics USD ----------------------------
        Field(
            alias='selling_price_usd',
            definition=Sum(fr.selling_price_usd),
            label='Total GMV (USD)',
            precision=2,
            thousands=','
        ),
        Field(
            alias='gross_revenue_usd',
            definition=Sum(fr.gross_revenue_usd),
            label='Gross Revenue (USD)',
            precision=2,
            thousands=','
        ),
        Field(
            alias='gross_profit_usd',
            definition=Sum(fr.gross_profit_usd),
            label='Gross Profit (USD)',
            precision=2,
            thousands=','
        ),
        Field(
            alias='gross_profit_granted_usd',
            definition=Sum(fr.gross_profit_granted_usd),
            label='Gross Profit Granted (USD)',
            precision=2,
            thousands=','
        ),
        # Cashback   ----------------------------------------
        Field(
            alias='breakage',
            definition=Sum(pcg.remaining_amount),
            label='Breakage',
            precision=2,
            thousands=',',
        ),
        Field(
            alias='breakage_usd',
            definition=Sum(pcg.remaining_amount / fr.to_usd_exc_rate_on_rsv_date),
            label='Breakage (USD)',
            precision=2,
            thousands=',',
        ),
        Field(
            alias='cashback_utilization_rate',
            definition=Sum(pcg.initial_amount - pcg.remaining_amount) / Sum(pcg.initial_amount),
            label='Cashback Utilization Rate',
            precision=2,
        ),
        # User Profiling ------------------------------------
        Field(
            alias='likely_gender',
            definition=up.likely_gender,
            data_type=DataType.text,
            label='Gender'
        ),
        Field(
            alias='age',
            definition=up.age,
            label='Age'
        ),
        # Cohort Analysis -----------------------------------
        Field(
            alias='cohort_month',
            definition=(
                DatePart('month', fr2.reservation_date_lt) 
                - DatePart('month', fr.reservation_date_lt) 
                + 12 * (DatePart('year', fr2.reservation_date_lt) - DatePart('year', fr.reservation_date_lt))
            ),
            label='Cohort Month'
        ),
        Field(
            alias='user_activities_type',
            definition=fr2.reservation_type,
            data_type=DataType.text,
            label='User Activities Type'
        ),
    ]
)